#!/usr/bin/env python
# encoding: utf-8
"""
@CreateTime: 2021/8/10 14:48
@Author: lxc
@LastEditTime:
@Desctiption: 更新 update_code_period(设备传输周期表) 中的 设备码
"""

import configparser

import pymysql

import LogOut
from mysqlDataBase import MysqldbOperational


def main():
    """
    更新设备名和设备码
    :return: SUCCESS 或 ERROR
    """
    try:
        # 点表中的设备名和设备码
        point_sql = "SELECT DISTINCT SUBSTRING_INDEX(mqtt_code, '.',1) FROM data_point_tbl WHERE mqtt_code IS NOT NULL;"
        point_code = cameraSql.executeSql(point_sql)
        # 设备传输周期表中的设备名和设备码
        period_code = cameraSql.executeSql("SELECT code FROM update_code_period;")
    except pymysql.Error as err:
        _logger.error(repr(err))

    point_diff = set(point_code).difference(set(period_code))     # 点表中的不同部分
    period_diff = set(period_code).difference(set(point_code))    # 设备传输周期表中的不同部分
    # noinspection PyBroadException
    try:
        # 删除 设备传输周期表中 不同的部分
        if len(period_diff) > 0:
            for each in period_diff:
                cond_dict = {"code": each[0]}
                cameraSql.delete(tablename="update_code_period", cond_dict=cond_dict)
        # 向 设备传输周期表 中添加 点表 中的不同部分
        if len(point_diff) > 0:
            code = []
            for each in point_diff:
                code.append("(\'"+each[0]+"\')")
            cameraSql.insert_sql(table_name="update_code_period",
                                 keys="(code)",
                                 values=code)
        # 修改新加设备码的上传周期
        cameraSql.executeCommit("UPDATE update_code_period SET period=2 WHERE period is NULL;")
        _logger.info("SUCCESS")
    except Exception as e:
        _logger.error("ERROR %s" % (e))




if __name__ == '__main__':
    _logger = LogOut.Log('update_code')

    # 创建读取配置文件对象
    config = configparser.ConfigParser()
    config.read("config.ini", encoding="utf-8")

    # 获取通用配置项
    GENERAL = "General"   # 读取标签
    mysql_host = config.get(GENERAL, 'mysqlHost')
    mysql_username = config.get(GENERAL, 'mysqlUsername')
    mysql_password = config.get(GENERAL, 'mysqlPassword')
    mysql_port = config.get(GENERAL, 'mysqlPort')

    # 获取特有配置项
    SECTION = 'updateDeviceUploadPeriod'
    mysql_database = config.get(SECTION, 'mysqlDatabase')
    cameraSql = MysqldbOperational(host=mysql_host,
                                   username=mysql_username,
                                   password=mysql_password,
                                   port=int(mysql_port),
                                   database=mysql_database,
                                   logger=_logger)

    main()
