#!/usr/bin/env python
# encoding: utf-8
"""
@CreateTime: 2021/9/26 16:25
@Author: lxc
@LastEditTime: 
@Desctiption: 上传生物监控系统数据到云平台
"""

import time
import LogOut
import configparser
from mysqlDataBase import MysqldbOperational
import requests
import json


def post_data(all_data):
    """
    上传生物监控子系统的数据到云平台
    :param datas:
    :return:
    """
    now = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
    success_date = []
    headers = {
        "Authorization": token,
        "Content-Type": "application/json"
    }
    for each in all_data:
        data = all_data[each]
        data["platformId"] = platform_id
        data["cageId"] = cage_id
        data["date"] = each
        data["upperRange"] = upper_range
        data["middleRange"] = middle_range
        data["lowerRange"] = lower_range
        if 'cageBiomass' in data.keys():
            data["breedBiomass"] = int(coefficient) * data['cageBiomass']

        if 'breedInfo' not in data.keys():
            data['breedInfo'] = []
        print("data = ", data)

        url = "https://management.super-sight.com.cn/system/api/v1/farmingOperationLog/log/monitor"
        ret = requests.post(url=url, headers=headers, json=data, verify=False)
        if ret.status_code == 200:
            print(now, ret.text, ret.status_code)
            success_date.append(each)
        else:
            print(now, ret.text, ret.status_code)
    if len(success_date) > 0:
        change_upload_status(success_date)


def deal_data():
    data_all = {}

    # 声呐的数据；每天的总数
    sonar_data_dailyReport = sonarSql.sonar_data_dailyReport()
    print(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())), 'sonar_data_dailyReport = ',
          sonar_data_dailyReport)
    if len(sonar_data_dailyReport) > 0:
        for each in sonar_data_dailyReport:
            date = each["date"]
            each_data = {"cageBiomass": each["assess_total_number"]}
            data_all[date] = each_data

    # 声呐的数据；每天的上中下三层的均值
    sonar_data_avg = sonarSql.sonar_data_avg()
    print(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())), 'sonar_data_avg = ', sonar_data_avg)
    if len(sonar_data_avg) > 0:
        for each in sonar_data_avg:
            date = each["date"]
            each_data = {"upperBiomass": each["assess_top_number"], "middleBiomass": each["assess_middle_number"],
                         "lowerBiomass": each["assess_bottom_number"]}
            if date in data_all.keys():
                tmp = data_all[date]
                each_data = {**tmp, **each_data}
            data_all[date] = each_data

    # 双目的数据
    binocular_datas = binocularSql.binocular_data()
    print(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())), 'binocular_datas = ', binocular_datas)
    if len(binocular_datas) > 0:
        for each in binocular_datas:
            breed_info = {"breedName": each["fish_species"], "lengthAverage": each["size_average"],
                          "lengthMedian": each["size_median"], "weightAverage": each["weight_average"],
                          "weightMedian": each["weight_median"]}
            # breed_info["breedBiomass"] =

            date = str(each["datetime"])
            if date in data_all.keys():
                tmp = data_all[date]
                if "breedInfo" not in tmp.keys():
                    tmp["breedInfo"] = [breed_info]
                else:
                    tmp["breedInfo"].append(breed_info)
                data_all[date] = tmp
            else:
                data_all[date] = {"breedInfo": [breed_info]}

    # 将数据上传至云平台
    if len(data_all) > 0:
        post_data(data_all)


def change_upload_status(date):
    """
    修改数据的上传状态
    :param date: 上传成功的数据的时间列表：['2021-07-06', '2021-07-07', '2021-07-08', ...]
    :return:
    """
    try:
        upload_binocular = binocularSql.binocular_send_status(date)
        upload_sonar = sonarSql.sonar_send_status(date)
    except Exception as e:
        print(e)


if __name__ == '__main__':
    time.sleep(2)
    _logger = LogOut.Log('upload_biology_monitor_data')

    run_time = 0
    file_time = 0
    while True:
        now_time = time.time()
        if now_time - run_time > 24 * 60 * 60:
            run_time = now_time

            # 创建读取配置文件对象
            config = configparser.ConfigParser()
            config.read("config.ini", encoding="utf-8")

            token = config.get("General", "token")
            platform_id = config.get("General", "appid")  # 平台ID

            # 双目和声呐数据库：获取通用配置项
            SECTION = "biologyMonitor"  # 读取的SECTION标签
            mysql_host = config.get(SECTION, 'mysqlHost')
            mysql_username = config.get(SECTION, 'mysqlUsername')
            mysql_password = config.get(SECTION, 'mysqlPassword')
            mysql_port = config.getint(SECTION, 'mysqlPort')
            binocular_database = config.get(SECTION, 'binocular_database')  # 双目数据库
            sonar_database = config.get(SECTION, 'sonar_database')  # 声呐数据库
            cage_id = config.get(SECTION, "cageId")  # 网箱ID
            upper_range = config.get(SECTION, "upperRange")  # 上层范围
            middle_range = config.get(SECTION, "middleRange")  # 中层范围
            lower_range = config.get(SECTION, "lowerRange")  # 下层范围
            coefficient = config.get(SECTION, "coefficient")  # 系数：来计算各种鱼的数量

            binocularSql = MysqldbOperational(host=mysql_host,
                                              username=mysql_username,
                                              password=mysql_password,
                                              port=mysql_port,
                                              database=binocular_database,
                                              logger=_logger)

            sonarSql = MysqldbOperational(host=mysql_host,
                                          username=mysql_username,
                                          password=mysql_password,
                                          port=mysql_port,
                                          database=sonar_database,
                                          logger=_logger)
            deal_data()
